@@ -11,7 +11,7 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,s
    cl <- match.call()
    n <- dim(r)[2]
   
-   if (!isCorrelation(r)) {
+   if (!isCorrelation(r)  && (!isCovariance(r))){ #added  (isCovariance)  April 9, 2019 to handle the case of a covariance matrix
                     raw <- TRUE
       				n.obs <- dim(r)[1] 
       				if(scores) {x.matrix <- as.matrix(r)  #matrices are required for the substitution to work
@@ -163,7 +163,7 @@ switch(rotate,  #The orthogonal cases  for GPArotation + ones developed for psyc
      				    Phi <- NULL} else { 
      				      
      				             ob <- try(do.call(getFromNamespace(rotate,'GPArotation'),list(loadings,...)))
-     				               if(class(ob)== as.character("try-error"))  {warning("The requested transformaton failed, Promax was used instead as an oblique transformation")
+     				               if(inherits(ob, as.character("try-error")))  {warning("The requested transformaton failed, Promax was used instead as an oblique transformation")
      				               ob <- Promax(loadings)}
      				                 
      				loadings <- ob$loadings
@@ -247,7 +247,7 @@ switch(rotate,  #The orthogonal cases  for GPArotation + ones developed for psyc
    
             
                result$weights <- try(solve(r,result$Structure),silent=TRUE)
-               if(class(result$weights) == "try-error")  {warning("The matrix is not positive semi-definite, scores found from Structure loadings") 
+               if(inherits(result$weights, "try-error"))  {warning("The matrix is not positive semi-definite, scores found from Structure loadings") 
                result$weights <- result$Structure} # else {  
                result$scores <- scale(x.matrix,scale=!covar) %*% result$weights #}
             }
