@@ -6,13 +6,24 @@
 #modifed November 3, 2018 to allow a single DV (in case cohen.d is just comparing two groups on one DV)
 #corrected January 1, 2019 to allow for grouping variables that are characters
 "statsBy" <-
-   function (data,group,cors=FALSE, cor="cor", method="pearson",use="pairwise", poly=FALSE,na.rm=TRUE,alpha=.05) { #  
-    cl <- match.call()
-  valid <- function(x) { #count the number of valid cases 
+   function (data,
+            group,
+            cors=FALSE,
+            cor="cor",
+             method="pearson",
+             use="pairwise", 
+             poly=FALSE,
+             na.rm=TRUE,
+             alpha=.05,
+             minlength=5) { #  
+ cl <- match.call()
+    
+valid <- function(x) { #count the number of valid cases 
         sum(!is.na(x))
     }
+    
 #define a function to count pairwise observations
-  pairwise <- function(x) {n <- t(!is.na(x)) %*% (!is.na(x))
+ pairwise <- function(x) {n <- t(!is.na(x)) %*% (!is.na(x))
               n}
 #get the grouping information
 
@@ -32,17 +43,22 @@ z1 <- data[,group]
        xvals <- list()
        #find the statistics by group
                temp <- by(data,z,colMeans,na.rm=na.rm)
-               
-               rownn <- lapply(temp,is.null)
+               rowname <- dimnames(temp)[[1]]
+               if(length(dimnames(temp))> 1){            #if we have multiple criteria, we need to name them
+               for (i in 2:length(dimnames(temp))) {
+               rowname <- paste0(rep(rowname,each=length(dimnames(temp)[[i]])),"-",dimnames(temp)[[i]])} 
+               }
+
+               rownn <- lapply(temp,is.null)     #drop rows without values
                if(sum(as.integer(rownn)) > 0) {
-              	 rown <-  names(temp)[-which(rownn==TRUE)] } else {rown <- names(temp) }              
+              	 rowname <-  rowname[-which(rownn==TRUE)] }  # look for missing criteria           
                xvals$mean <- t(matrix(unlist(temp),nrow=ncol(data)))              
                xvals$sd <-t(matrix(unlist(by(data,z,function(x) sapply(x,sd,na.rm=na.rm))),nrow=ncol(data)))
                xvals$n <- t(matrix(unlist(by(data,z,function(x) sapply(x,valid))),nrow=ncol(data)))
               
                
                colnames(xvals$mean) <- colnames(xvals$sd) <- colnames(xvals$n) <-  colnames(data)
-               rownames(xvals$mean) <-  rownames(xvals$sd) <- rownames(xvals$n) <- rown
+               rownames(xvals$mean) <-  rownames(xvals$sd) <- rownames(xvals$n) <- rowname
                              # nH <- harmonic.mean(xvals$n)     #this will be 0 if any is zero -- but is not used anyway
                nG <- colSums(!is.na(xvals$mean))         #fixed this so it is just for the cells that are not NA
                GM <- colSums(xvals$mean*xvals$n,na.rm=na.rm)/colSums(xvals$n,na.rm=na.rm) 
@@ -93,24 +109,32 @@ z1 <- data[,group]
        poly = {r <- by(data,z,function(x) polychoric(x[-gr])$rho)},
        mixed = {r <- by(data,z,function(x) mixed.cor(x[-gr])$rho)}
        )         
-                 
+            
+             nWg <- by(data,z,function(x) pairwise(x[-gr, -gr]))
               nvars <-  ncol(r[[1]])
               xvals$r <- r   #store them as square matrices
-              lower <- lapply(r,function(x) x[lower.tri(x)])
+             
+              length.r <- length(r)
+              # xvals$r.ci <- r
+             
+              xvals$r.ci <- mapply(cor.Ci, r=r, n=nWg)
+              attributes(xvals$r.ci) <- attributes(r)
+              lower <- lapply(r,function(x) if(!is.null(x)){ x[lower.tri(x)]})
               xvals$within <- t(matrix(unlist(lower),nrow=nvars*(nvars-1)/2))  #string them out as well
              
-              cnR <- abbreviate(cnames[-gr],minlength=5) 
+              cnR <- abbreviate(cnames[-gr],minlength=minlength) 
            
       k <- 1
        colnames(xvals$within) <- paste("V",1:ncol(xvals$within))
       for(i in 1:(nvars-1)) {for (j in (i+1):nvars) {
       	colnames(xvals$within)[k] <- paste(cnR[i],cnR[j],sep="-")
-     	 k<- k +1 }} 
-     	 rownames(xvals$within) <- paste0("z",names(xvals$r))
-
+     	 k<- k +1 }}
+     	  
+     	# rownames(xvals$within) <- paste0("z",names(xvals$r))
+          rownames(xvals$within) <- rowname
              
              wt <- by(data,z,function(x) pairwiseCount(x[-gr]))
-             lower.wt <- t(matrix(unlist(lapply(wt,function(x) x[lower.tri(x)])    )  ,nrow=nvars*(nvars-1)/2))
+             lower.wt <- t(matrix(unlist(lapply(wt,function(x) if(!is.null(x)) { x[lower.tri(x)]})    )  ,nrow=nvars*(nvars-1)/2))
              lower.wt <- t(t(lower.wt)/colSums(lower.wt,na.rm=TRUE))
              pool  <- colSums( lower.wt * xvals$within,na.rm=TRUE)
              pool.sd <- apply(xvals$within, 2,FUN=sd, na.rm=TRUE)
@@ -141,11 +165,13 @@ z1 <- data[,group]
              new.data <- as.matrix( merge(xvals$mean,data,by=group,suffixes =c(".bg",""))) #drop the grouping variable(s) 
              new.data <- new.data[,(length(group)+1):ncol(new.data)]
              
-             diffs <- new.data[,(nvar+1):ncol(new.data),drop=FALSE] - new.data[,1:nvar]
-             
+             diffs <- new.data[,(nvar+1):ncol(new.data),drop=FALSE] - new.data[,1:nvar]  #Difference of raw data within group - within group mean
+             #why don't we just use scale for the data within each group?
 
              colnames(diffs) <- paste(colnames(new.data)[(nvar + 1):ncol(new.data)], ".wg", sep = "")
              if(nvar > 1) {xvals$rbg <- cor(new.data[,1:nvar],use="pairwise",method=method)} else {xvals$rbg <- NA}  #the between group (means)
+             nBg <- pairwise(xvals$mean)[-gr,-gr]
+            if(any(nBg > 2)) {xvals$ci.bg <-cor.Ci(xvals$rbg,nBg,alpha=alpha,minlength=minlength)} else {xvals.ci.bg=NA}  #added the test for all nBg  <= 2 so we don't call cor.Ci and throw an error  11/2/19
              #t <- rep(NA,(length(nG)-length(gr)))
              
              if(all(nG > 2)) {
@@ -162,9 +188,10 @@ z1 <- data[,group]
               covar <- TRUE}
        )
        
-             xvals$nw <- pairwise(diffs)
+               xvals$nw <- pairwise(diffs)
                rwg <- cov2cor(xvals$rwg)
                t <- (rwg*sqrt(xvals$nw -2))/sqrt(1-rwg^2)
+               xvals$ci.wg <- cor.Ci(rwg,xvals$nw,alpha=alpha,minlength=minlength)
                
                
             if(all(nG > 2)) {
@@ -176,8 +203,31 @@ z1 <- data[,group]
          
             xvals$nwg <- N - nG
             xvals$nG <- nG
+            
             xvals$Call <- cl
     statsBy <- xvals
     class(statsBy) <- c("psych","statsBy")
     return(statsBy)
     }
+    
+cor.Ci <- function(r,n,alpha=.05,minlength=10) {
+   cl <- match.call()
+
+z.r <- fisherz(r)
+sd <-1/sqrt(n-3)
+nvar <- NCOL(r)
+ci.lower <- fisherz2r(z.r  +( qnorm(alpha/2) * sd))
+ci.upper <- fisherz2r(z.r + (qnorm(1-alpha/2) * sd))
+cnR <- abbreviate(colnames(r),minlength=minlength)
+ci.lower <- ci.lower[lower.tri(ci.lower)]
+ci.upper <- ci.upper[lower.tri(ci.upper)]
+r.ci <- data.frame(lower=ci.lower,r=r[lower.tri(r)],upper=ci.upper)
+k <- 1
+ for(i in 1:(nvar-1)) {for (j in (i+1):nvar) {
+      rownames(r.ci)[k] <- paste(cnR[i],cnR[j],sep="-")
+ k <- k + 1     
+}}
+result <-list(r.ci=r.ci)
+class(result) <- cs(psych,corCi )
+return(result)
+}
